@@ -12,11 +12,15 @@
 #'
 #' @param spotConfig    the list of configurations for \link{SPOT}. Besides the usual \link{SPOT} settings,
 #'    this list has to contain an element \code{tdm} with the mandatory elements \itemize{
+#'      \item tdm$mainFunc:     the function name of the DM task to execute (e.g. "main_sonar")
 #'      \item tdm$mainFile:     the R file of the DM task to source
-#'      \item tdm$mainCommand:  the R command to execute, usually \code{result <- main_TASK(opts)}. 
+#'      \item tdm$mainCommand:  the R command to execute, usually \code{result <- <mainFunc>(opts,dset=dset)} where <mainFunc> is the string in tdm$mainFunc. 
 #'          (It is expected that \code{mainCommand} returns \code{result} and that the element 
 #'          \code{result$y} contains the quantity to be minimized by \link{SPOT}.)
 #'      }
+#'    Optionally, \code{tdm} may define  \itemize{
+#'      \item tdm$mainFile:     e.g. "myDir/main_sonar.r", the R file of the DM task to source and \code{dirname(tdm$mainFile)} is the directory where tdm$mainFunc is executed.
+#'      }
 #' @return spotConfig
 #'
 #' @seealso \code{\link{spot}}, \code{\link{spotTuner}}, \code{\link{lhdTuner}}, \code{\link{tdmDispatchTuner}}
@@ -25,14 +29,18 @@
 #' @export
 ######################################################################################
 tdmStartSpot <- function(spotConfig) {
+	SAVESEED<-.Random.seed	#save the Random Number Generator RNG status
     if (!is.list(spotConfig)) stop("Error: spotConfig is not a list");
     if (is.null(spotConfig$opts)) stop("Error: spotConfig does not contain an element 'opts'");
     if (is.null(spotConfig$tdm)) stop("Error: spotConfig does not contain an element 'tdm'");
     if (is.null(spotConfig$tdm$fileMode)) spotConfig$tdm$fileMode <- TRUE;
         # This default setting is useful to allow a simpler TDM-Phase-2 call (with tdm$fileMode not def'd).
-        # However, tdm has to specify the mandatory settings tdm$mainFile, tdm$mainCommand
+        # However, tdm has to specify the mandatory setting: either tdm$mainFunc or  tdm$mainFile + tdm$mainCommand
     tdm <- spotConfig$tdm;
     opts <- spotConfig$opts;
+    dset <- switch(as.character(is.null(spotConfig$dataObj)),"TRUE"=NULL,"FALSE"=dsetTrnVa(spotConfig$dataObj));    
+    # If dset is not NULL, this has an effect on tdm$mainCommand which contains "...,dset=dset".
+    # If dset is NULL, the data reading is deferred to main_TASK.
 
   	writeLines("tdmStartSpot run...", con=stderr());   
 # --- this is now for phase 3 in tdmCompleteEval.r (or for phase 2 in appropriate phase2-script) ---
@@ -41,7 +49,7 @@ tdmStartSpot <- function(spotConfig) {
 #  	## read default problem design  (here: set default values for all elements of list opts)
 #  	source(pdFile);                  # contains *no longer* the definition of tdm$mainFile & tdm$mainCommand
 #
-#    source(tdm$mainFile);          
+#   if (!is.null(tdm$mainFile")  source(tdm$mainFile);          
 
   	## read doe/dace etc settings:
   	if (spotConfig$spot.fileMode) {
@@ -76,11 +84,12 @@ tdmStartSpot <- function(spotConfig) {
       	opts <- tdmMapDesSpot$apply(des,opts,k,tdm);
    			
         if (!is.null(des$STEP))	theStep <- des$STEP[k];
-  			seed <- des$SEED[k]+r;		# probably not used	
+  			opts$ALG.SEED <- des$SEED[k]+r;		# now used in tdmClassify, tdmClassifyLoop
   			
   			cat(sprintf("Config: %5d,   Repeat: %5d\n",des$CONFIG[k],r));
   			
-  			oldwd = getwd(); setwd(dirname(tdm$mainFile));    # save & change working dir 		         			
+  			oldwd = getwd(); 
+        if (!is.null(tdm$mainFile)) setwd(dirname(tdm$mainFile));    # save & change working dir 		         			
     		result = NULL;     		
         eval(parse(text=tdm$mainCommand));                # execute the command given in string tdm$mainCommand
         if (is.null(result$y)) stop("tdm$mainCommand did not return a list 'result' containing an element 'y'");
@@ -91,7 +100,7 @@ tdmStartSpot <- function(spotConfig) {
                               ,des[k,setdiff(names(des),c("CONFIG","REPEATS","repeatsLastConfig","STEP","SEED"))]
                               ));
         res <- cbind(res
-          					,SEED=seed
+          					,SEED=opts$ALG.SEED
          					  ,STEP=theStep
           					,CONFIG=des$CONFIG[k]                  
           					,REP=r
@@ -113,7 +122,7 @@ tdmStartSpot <- function(spotConfig) {
   			}
   		}	# for (r)			
   	}	# for (k)	
- 	
+ 	assign(".Random.seed", SAVESEED, envir=globalenv()); 		#load the saved RNG status
   	return(spotConfig);            # new, *necessary* !!
 }
 
